@@ -683,8 +683,6 @@ static void load_events(op_cpu cpu_type)
 {
 	const char * cpu_name = op_get_cpu_name(cpu_type);
 	struct list_head * pos;
-	struct op_event *event;
-	struct op_unit_mask *unit_mask;
 	int err = 0;
 
 	if (!list_empty(&events_list))
@@ -702,57 +700,6 @@ static void load_events(op_cpu cpu_type)
 	if (err)
 		exit(err);
 
-	if (!op_cpu_has_timer_fs())
-		return;
-
-	/* sanity check: Don't use event `TIMER' since it is predefined.  */
-	list_for_each(pos, &events_list) {
-		struct op_event * event = list_entry(pos, struct op_event,
-						     event_next);
-
-		if (strcmp(event->name, TIMER_EVENT_NAME) == 0) {
-			fprintf(stderr, "Error: " TIMER_EVENT_NAME
-				" event cannot be redefined.\n");
-			exit(EXIT_FAILURE);
-		}
-		if (event->val == TIMER_EVENT_VALUE) {
-			fprintf(stderr, "Error: Event %s uses " TIMER_EVENT_NAME
-				" which is reserverd for timer based sampling.\n",
-				event->name);
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	list_for_each(pos, &um_list) {
-		struct op_unit_mask * um = list_entry(pos, struct op_unit_mask,
-						      um_next);
-		if (strcmp(um->name, TIMER_EVENT_UNIT_MASK_NAME) == 0) {
-			fprintf(stderr, "Error: " TIMER_EVENT_UNIT_MASK_NAME
-				" unit mask cannot be redefined.\n");
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	unit_mask = new_unit_mask();
-	unit_mask->name = xstrdup(TIMER_EVENT_UNIT_MASK_NAME);
-	unit_mask->num = 1;
-	unit_mask->unit_type_mask = utm_mandatory;
-	unit_mask->um[0].extra = 0;
-	unit_mask->um[0].value = 0;
-	unit_mask->um[0].name = xstrdup("");
-	unit_mask->um[0].desc = xstrdup("No unit mask");
-	unit_mask->used = 1;
-
-	event = new_event();
-	event->name = xstrdup(TIMER_EVENT_NAME);
-	event->desc = xstrdup(TIMER_EVENT_DESC);
-	event->val = TIMER_EVENT_VALUE;
-	event->unit = unit_mask;
-	event->min_count = 0;
-	event->filter = 0;
-	event->counter_mask = 1 << (op_get_nr_counters(cpu_type) - 1);
-	event->ext = NULL;
-	event->filter = -1;
 }
 
 struct list_head * op_events(op_cpu cpu_type)
@@ -948,7 +895,6 @@ char const * find_mapping_for_event(u32 nr, op_cpu cpu_type)
 	FILE * fp = open_event_mapping_file(cpu_name);
 	char const * map = NULL;
 	switch (cpu_type) {
-		case CPU_PPC64_PA6T:
 		case CPU_PPC64_970:
 		case CPU_PPC64_970MP:
 		case CPU_PPC64_POWER4:
@@ -957,7 +903,6 @@ char const * find_mapping_for_event(u32 nr, op_cpu cpu_type)
 		case CPU_PPC64_POWER5pp:
 		case CPU_PPC64_POWER6:
 		case CPU_PPC64_POWER7:
-		case CPU_PPC64_IBM_COMPAT_V1:
 		// For ppc64 types of CPU_PPC64_ARCH_V1 and higher, we don't need an event_mappings file
 			if (!fp) {
 				fprintf(stderr, "oprofile: could not open event mapping file %s\n", filename);
@@ -1201,6 +1146,8 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
  		case CPU_CORE_I7:
 		case CPU_NEHALEM:
 		case CPU_HASWELL:
+		case CPU_BROADWELL:
+		case CPU_SILVERMONT:
 		case CPU_WESTMERE:
 		case CPU_SANDYBRIDGE:
 		case CPU_IVYBRIDGE:
@@ -1212,30 +1159,15 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 			descr->name = "CPU_CLK_UNHALTED";
 			break;
 
-		case CPU_RTC:
-			descr->name = "RTC_INTERRUPTS";
-			descr->count = 1024;
-			break;
-
 		case CPU_P4:
 		case CPU_P4_HT2:
 			descr->name = "GLOBAL_POWER_EVENTS";
 			descr->um = 0x1;
 			break;
 
-		case CPU_IA64:
-		case CPU_IA64_1:
-		case CPU_IA64_2:
-			descr->count = 1000000;
-			descr->name = "CPU_CYCLES";
-			break;
-
-		case CPU_AXP_EV4:
-		case CPU_AXP_EV5:
-		case CPU_AXP_PCA56:
-		case CPU_AXP_EV6:
 		case CPU_AXP_EV67:
 			descr->name = "CYCLES";
+			descr->um = 0x1;
 			break;
 
 		// we could possibly use the CCNT
@@ -1248,13 +1180,15 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 		case CPU_ARM_V7_CA7:
 		case CPU_ARM_V7_CA9:
 		case CPU_ARM_V7_CA15:
-		case CPU_AVR32:
 		case CPU_ARM_SCORPION:
 		case CPU_ARM_SCORPIONMP:
+		case CPU_ARM_KRAIT:
+		case CPU_ARM_V8_APM_XGENE:
+		case CPU_ARM_V8_CA57:
+		case CPU_ARM_V8_CA53:
 			descr->name = "CPU_CYCLES";
 			break;
 
-		case CPU_PPC64_PA6T:
 		case CPU_PPC64_970:
 		case CPU_PPC64_970MP:
 		case CPU_PPC_7450:
@@ -1263,9 +1197,7 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 		case CPU_PPC64_POWER6:
 		case CPU_PPC64_POWER5p:
 		case CPU_PPC64_POWER5pp:
-		case CPU_PPC64_CELL:
 		case CPU_PPC64_POWER7:
-		case CPU_PPC64_IBM_COMPAT_V1:
 		case CPU_PPC64_ARCH_V1:
 		case CPU_PPC64_POWER8:
 			descr->name = "CYCLES";
@@ -1308,6 +1240,8 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 
 		case CPU_PPC_E500:
 		case CPU_PPC_E500_2:
+		case CPU_PPC_E500MC:
+		case CPU_PPC_E6500:
 		case CPU_PPC_E300:
 			descr->name = "CPU_CLK";
 			break;
@@ -1315,14 +1249,9 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 		case CPU_S390_Z10:
 		case CPU_S390_Z196:
 		case CPU_S390_ZEC12:
- 			if (op_get_nr_counters(cpu_type) > 1) {
- 				descr->name = "HWSAMPLING";
- 				descr->count = 4127518;
- 			} else {
- 				descr->name = TIMER_EVENT_NAME;
- 				descr->count = 10000;
- 			}
-  			break;
+			descr->name = "CPU_CYCLES";
+			descr->count = 4127518;
+			break;
 
 		case CPU_TILE_TILE64:
 		case CPU_TILE_TILEPRO:
@@ -1347,8 +1276,8 @@ static void extra_check(struct op_event *e, char *name, unsigned w)
 	if (!e->unit->um[w].extra) {
 		fprintf(stderr,
 			"Named unit mask (%s) not allowed for event without 'extra:' values.\n"
-			"Please specify the numerical value for the unit mask. See 'opcontrol'\n"
-			"or 'operf' man page for more info.\n", name);
+			"Please specify the numerical value for the unit mask. See the 'operf'\n"
+			"man page for more info.\n", name);
 		exit(EXIT_FAILURE);
 	}
 
@@ -1383,6 +1312,7 @@ static void do_resolve_unit_mask(struct op_event *e,
 		if (pe->unit_mask_name == NULL) {
 			/* For numerical unit mask */
 			int found = 0;
+			int old_um_valid = pe->unit_mask_valid;
 
 			/* Use default unitmask if not specified */
 			if (!pe->unit_mask_valid) {
@@ -1398,9 +1328,16 @@ static void do_resolve_unit_mask(struct op_event *e,
 					found++;
 			}
 			if (found > 1) {
-				fprintf(stderr, "Unit mask (0x%x) is non unique.\n"
-				        "Please specify the unit mask using the first "
-					"word of the description\n",
+				if (!old_um_valid)
+					fprintf(stderr,
+						"Default unit mask not supported for this event.\n"
+						"Please speicfy a unit mask by name, using the first "
+						"word of the unit mask description\n");
+				else
+					fprintf(stderr,
+						"Unit mask (0x%x) is non unique.\n"
+						"Please specify the unit mask using the first "
+						"word of the description\n",
 					pe->unit_mask);
 				exit(EXIT_FAILURE);
 			}
