@@ -52,7 +52,9 @@ extern op_cpu cpu_type;
 
 using namespace std;
 
-static int _op_get_next_online_cpu(DIR * dir, struct dirent *entry)
+// Global functions
+
+int op_pe_utils::op_get_next_online_cpu(DIR * dir, struct dirent *entry)
 {
 #define OFFLINE 0x30
 	unsigned int cpu_num;
@@ -86,8 +88,6 @@ static int _op_get_next_online_cpu(DIR * dir, struct dirent *entry)
 		return cpu_num;
 }
 
-// Global functions
-
 int op_pe_utils::op_get_sys_value(const char * filename)
 {
 	char str[10];
@@ -148,7 +148,7 @@ int op_pe_utils::op_get_cpu_for_perf_events_cap(void)
 			goto error;
 		} else {
 			struct dirent *entry = NULL;
-			retval = _op_get_next_online_cpu(dir, entry);
+			retval = op_get_next_online_cpu(dir, entry);
 			closedir(dir);
 		}
 	} else {
@@ -310,40 +310,6 @@ int op_pe_utils::op_validate_app_name(char ** app, char ** save_appname)
 
 	out: return rc;
 }
-static int _get_next_online_cpu(DIR * dir, struct dirent *entry)
-{
-#define OFFLINE 0x30
-	unsigned int cpu_num;
-	char cpu_online_pathname[40];
-	int res;
-	FILE * online;
-	again:
-	do {
-		entry = readdir(dir);
-		if (!entry)
-			return -1;
-	} while (entry->d_type != DT_DIR);
-
-	res = sscanf(entry->d_name, "cpu%u", &cpu_num);
-	if (res <= 0)
-		goto again;
-
-	errno = 0;
-	snprintf(cpu_online_pathname, 40, "/sys/devices/system/cpu/cpu%u/online", cpu_num);
-	if ((online = fopen(cpu_online_pathname, "r")) == NULL) {
-		cerr << "Unable to open " << cpu_online_pathname << endl;
-		if (errno)
-			cerr << strerror(errno) << endl;
-		return -1;
-	}
-	res = fgetc(online);
-	fclose(online);
-	if (res == OFFLINE)
-		goto again;
-	else
-		return cpu_num;
-}
-
 
 set<int> op_pe_utils::op_get_available_cpus(int max_num_cpus)
 {
@@ -368,8 +334,7 @@ set<int> op_pe_utils::op_get_available_cpus(int max_num_cpus)
 		goto out;
 	}
 	memset(cpus_online, 0, sizeof(cpus_online));
-	fgets(cpus_online, sizeof(cpus_online), online_cpus);
-	if (!cpus_online[0]) {
+	if (fgets(cpus_online, sizeof(cpus_online), online_cpus) == NULL) {
 		fclose(online_cpus);
 		err_msg = "Internal Error: Number of online cpus cannot be determined.";
 		rc = -1;
@@ -392,7 +357,7 @@ set<int> op_pe_utils::op_get_available_cpus(int max_num_cpus)
 		if (all_cpus_avail) {
 			available_cpus.insert(cpu);
 		} else {
-			real_cpu = _get_next_online_cpu(dir, entry);
+			real_cpu = op_get_next_online_cpu(dir, entry);
 			if (real_cpu < 0) {
 				err_msg = "Internal Error: Number of online cpus cannot be determined.";
 				rc = -1;
@@ -447,8 +412,8 @@ static void _get_event_code(operf_event_t * event, op_cpu cpu_type)
 
 
 #if defined(__i386__) || defined(__x86_64__)
+	char mask[OP_MAX_UM_NAME_LEN];
 	// Setup EventSelct[11:8] field for AMD
-	char mask[12];
 	const char * vendor_AMD = "AuthenticAMD";
 	if (op_is_cpu_vendor((char *)vendor_AMD)) {
 		config = base_code & 0xF00ULL;
@@ -518,9 +483,10 @@ handle_named_um:
 		pclose(fp);
 		event->evt_um = strtoull(mask, &endptr, 10);
 		if ((endptr >= mask) &&
-				(endptr <= (mask + strlen(mask) - 1))) {
+				(endptr <= (mask + strlen(mask) - 2))) { // '- 2' to account for linefeed and '\0'
+
 			// Must be a default named unit mask
-			strncpy(event->um_name, mask, OP_MAX_UM_NAME_LEN);
+			strncpy(event->um_name, mask, OP_MAX_UM_NAME_LEN - 1);
 			goto handle_named_um;
 		}
 		config |= ((event->evt_um & 0xFFULL) << 8);
@@ -663,6 +629,7 @@ static bool _get_codes_for_match(unsigned int pfm_idx, const char name[],
 {
 	unsigned int num_events = evt_vec->size();
 	int tmp_code, ret;
+	bool edge_detect = false;
 	char evt_name[OP_MAX_EVT_NAME_LEN];
 	unsigned int events_converted = 0;
 	for (unsigned int i = 0; i < num_events; i++) {
@@ -680,6 +647,25 @@ static bool _get_codes_for_match(unsigned int pfm_idx, const char name[],
 		} else {
 			strncpy(evt_name, event.name, strlen(event.name));
 		}
+
+		/* Events where the "_EDGE_COUNT" suffix has been appended to a
+		 * real native event name are pseudo events (events that have
+		 * not been formally defined in processor documentation), where
+		 * we wish to detect the rising edge of the real native event.
+		 * This "edge detection" technique is useful for events that normally
+		 * count the number of cycles that a particular condition is true.
+		 * Since libpfm does not know about pseudo events, we need to
+		 * convert them to their real native event equivalent, and then
+		 * set the "edge detect" bit (the LSB) in the event code.
+		 */
+		string evt = evt_name;
+		size_t edge_suffix_pos = evt.rfind("_EDGE_COUNT");
+		if (edge_suffix_pos != string::npos) {
+			evt = evt.substr(0, edge_suffix_pos);
+			strncpy(evt_name, evt.c_str(), evt.length() + 1);
+			edge_detect = true;
+		}
+
 		if (strncmp(name, evt_name, OP_MAX_EVT_NAME_LEN))
 			continue;
 		ret = pfm_get_event_code(pfm_idx, &tmp_code);
@@ -690,18 +676,22 @@ static bool _get_codes_for_match(unsigned int pfm_idx, const char name[],
 			throw runtime_error(msg);
 		}
 		event.evt_code = tmp_code;
+		// Setting LSB of code makes this a "rising edge detection" type of event
+		if (edge_detect)
+			event.evt_code |= 1;
 		(*evt_vec)[i] = event;
 		events_converted++;
 		cverb << vdebug << "Successfully converted " << event.name << " to perf_event code "
-		      << hex << tmp_code << endl;
+		      << hex << event.evt_code << endl;
 	}
 	return (events_converted == num_events);
 }
 #else
 static bool _op_get_event_codes(vector<operf_event_t> * evt_vec)
 {
-	int ret, i;
+	int ret;
 	unsigned int num_events = evt_vec->size();
+	bool edge_detect = false;
 	char evt_name[OP_MAX_EVT_NAME_LEN];
 	unsigned int events_converted = 0;
 	u64 code[1];
@@ -738,6 +728,24 @@ static bool _op_get_event_codes(vector<operf_event_t> * evt_vec)
 			strncpy(evt_name, event.name, strlen(event.name));
 		}
 
+		/* Events where the "_EDGE_COUNT" suffix has been appended to a
+		 * real native event name are pseudo events (events that have
+		 * not been formally defined in processor documentation), where
+		 * we wish to detect the rising edge of the real native event.
+		 * This "edge detection" technique is useful for events that normally
+		 * count the number of cycles that a particular condition is true.
+		 * Since libpfm does not know about pseudo events, we need to
+		 * convert them to their real native event equivalent, and then
+		 * set the "edge detect" bit (the LSB) in the event code.
+		 */
+		string evt = evt_name;
+		size_t edge_suffix_pos = evt.rfind("_EDGE_COUNT");
+		if (edge_suffix_pos != string::npos) {
+			evt = evt.substr(0, edge_suffix_pos);
+			strncpy(evt_name, evt.c_str(), evt.length() + 1);
+			edge_detect = true;
+		}
+
 		memset(&raw, 0, sizeof(raw));
 		ret = pfm_get_os_event_encoding(evt_name, PFM_PLM3, PFM_OS_NONE, &raw);
 		if (ret != PFM_SUCCESS) {
@@ -746,8 +754,10 @@ static bool _op_get_event_codes(vector<operf_event_t> * evt_vec)
 					"; cannot continue";
 			throw runtime_error(msg);
 		}
-
 		event.evt_code = raw.codes[0];
+		// Setting LSB of code makes this a "rising edge detection" type of event
+		if (edge_detect)
+			event.evt_code |= 1;
 		(*evt_vec)[i] = event;
 		events_converted++;
 		cverb << vdebug << "Successfully converted " << event.name << " to perf_event code "
@@ -759,9 +769,6 @@ static bool _op_get_event_codes(vector<operf_event_t> * evt_vec)
 
 static bool convert_event_vals(vector<operf_event_t> * evt_vec)
 {
-	unsigned int i, count;
-	char name[256];
-	int ret;
 	for (unsigned int i = 0; i < evt_vec->size(); i++) {
 		operf_event_t event = (*evt_vec)[i];
 		if (cpu_type == CPU_PPC64_POWER7) {
@@ -779,6 +786,10 @@ static bool convert_event_vals(vector<operf_event_t> * evt_vec)
 	}
 
 #if HAVE_LIBPFM3
+	unsigned int i, count;
+	char name[256];
+	int ret;
+
 	if (pfm_initialize() != PFMLIB_SUCCESS)
 		throw runtime_error("Unable to initialize libpfm; cannot continue");
 
@@ -803,7 +814,8 @@ static bool convert_event_vals(vector<operf_event_t> * evt_vec)
 
 
 
-void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
+void op_pe_utils::op_process_events_list(set<string> & passed_evts,
+                                         bool do_profiling, bool do_callgraph)
 {
 	string cmd = OP_BINDIR;
 
@@ -812,11 +824,13 @@ void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
 		     << OP_MAX_EVENTS << "." << endl;
 		exit(EXIT_FAILURE);
 	}
-	cmd += "/ophelp --check-events --ignore-count ";
-	for (unsigned int i = 0; i <  passed_evts.size(); i++) {
+	cmd += "/ophelp --check-events ";
+	if (!do_profiling)
+		cmd += "--ignore-count ";
+	for (set<string>::iterator it = passed_evts.begin(); it != passed_evts.end(); it++) {
 		FILE * fp;
 		string full_cmd = cmd;
-		string event_spec = passed_evts[i];
+		string event_spec = *it;
 
 #if PPC64_ARCH
 		// Starting with CPU_PPC64_ARCH_V1, ppc64 events files are formatted like
@@ -825,6 +839,8 @@ void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
 			event_spec = _handle_powerpc_event_spec(event_spec);
 #endif
 
+		if (do_callgraph)
+			full_cmd += " --callgraph=1 ";
 		full_cmd += event_spec;
 		fp = popen(full_cmd.c_str(), "r");
 		if (fp == NULL) {
@@ -836,14 +852,22 @@ void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
 			pclose(fp);
 			cerr << "Error retrieving info for event "
 			     << event_spec << endl;
+			if (do_callgraph)
+				cerr << "Note: When doing callgraph profiling, the sample count must be"
+				     << endl << "15 times the minimum count value for the event."  << endl;
 			exit(EXIT_FAILURE);
 		}
 		pclose(fp);
 		char * event_str = op_xstrndup(event_spec.c_str(), event_spec.length());
 		operf_event_t event;
+		memset(&event, 0, sizeof(event));
 		strncpy(event.name, strtok(event_str, ":"), OP_MAX_EVT_NAME_LEN - 1);
+		if (do_profiling)
+			event.count = atoi(strtok(NULL, ":"));
+		else
+			event.count = 0UL;
 		/* Event name is required in the event spec in order for
-		 * 'ophelp --check-events --ignore-count' to pass.  But since unit mask
+		 * 'ophelp --check-events' to pass.  But since unit mask
 		 *  and domain control bits are optional, we need to ensure the result of
 		 *  strtok is valid.
 		 */
@@ -854,27 +878,43 @@ void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
 		int place =  _OP_UM;
 		char * endptr = NULL;
 		event.evt_um = 0UL;
-		event.count = 0UL;
 		event.no_kernel = 0;
 		event.no_user = 0;
 		event.throttled = false;
+		event.mode_specified = false;
+		event.umask_specified = false;
 		memset(event.um_name, '\0', OP_MAX_UM_NAME_LEN);
+		memset(event.um_numeric_val_as_str, '\0', OP_MAX_UM_NAME_STR_LEN);
 		while ((info = strtok(NULL, ":"))) {
 			switch (place) {
 			case _OP_UM:
 				event.evt_um = strtoul(info, &endptr, 0);
+				event.umask_specified = true;
+
 				// If any of the UM part is not a number, then we
 				// consider the entire part a string.
 				if (*endptr) {
 					event.evt_um = 0;
 					strncpy(event.um_name, info, OP_MAX_UM_NAME_LEN - 1);
+				} else {
+					/* event.evt_um gets modified later,
+					 * save the specified number as a
+					 * string to output later.
+					 */
+					stringstream strs;
+					strs << "0x" << hex << event.evt_um;
+					strncpy(event.um_numeric_val_as_str,
+						(char *)strs.str().c_str(),
+                                                strs.str().length());
 				}
 				break;
 			case _OP_KERNEL:
+				event.mode_specified = true;
 				if (atoi(info) == 0)
 					event.no_kernel = 1;
 				break;
 			case _OP_USER:
+				event.mode_specified = true;
 				if (atoi(info) == 0)
 					event.no_user = 1;
 				break;
@@ -882,6 +922,37 @@ void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
 			place++;
 		}
 		free(event_str);
+
+#ifdef __s390__
+		if (do_profiling) {
+			if (strncmp(event.name, "CPU_CYCLES", strlen(event.name)) != 0) {
+				cerr << "Profiling with " << event.name << " is not supported." << endl
+				     << "Only CPU_CYCLES is allowed to use with operf." << endl;
+				exit(EXIT_FAILURE);
+			}
+		} else {
+			if (!event.no_kernel && event.no_user) {
+				cerr << "Counting for just the kernel is not supported." << endl
+				     << "Re-run the command and simply pass the event name " << endl
+				     << "(" << event.name << ") for the event spec, without" << endl
+				     << "unit mask/kernel/user bits." << endl;
+				exit(EXIT_FAILURE);
+			}
+		}
+#endif
+
+#ifdef __alpha__
+		// Alpha arch does not support any mode exclusion, so if either user or kernel
+		// mode are excluded by the user, we'll exit with an error message.
+		if (event.no_kernel || event.no_user) {
+			cerr << "Mode exclusion is not supported on Alpha." << endl
+			     << "Re-run the command and simply pass the event name " << endl
+			     << "(" << event.name << ") for the event spec, without" << endl
+			     << "unit mask/kernel/user bits." << endl;
+			exit(EXIT_FAILURE);
+		}
+#endif
+
 		_get_event_code(&event, cpu_type);
 		events.push_back(event);
 	}
@@ -904,7 +975,7 @@ void op_pe_utils::op_process_events_list(vector<string> & passed_evts)
 #endif
 }
 
-void op_pe_utils::op_get_default_event(void)
+void op_pe_utils::op_get_default_event(bool do_callgraph)
 {
 	operf_event_t dft_evt;
 	struct op_default_event_descr descr;
@@ -918,7 +989,18 @@ void op_pe_utils::op_get_default_event(void)
 	}
 
 	memset(&dft_evt, 0, sizeof(dft_evt));
-	dft_evt.count = descr.count;
+	if (do_callgraph) {
+		struct op_event * _event;
+		op_events(cpu_type);
+		if ((_event = find_event_by_name(descr.name, 0, 0))) {
+			dft_evt.count = _event->min_count * CALLGRAPH_MIN_COUNT_SCALE;
+		} else {
+			cerr << "Error getting event info for " << descr.name << endl;
+			exit(EXIT_FAILURE);
+		}
+	} else {
+		dft_evt.count = descr.count;
+	}
 	dft_evt.evt_um = descr.um;
 	strncpy(dft_evt.name, descr.name, OP_MAX_EVT_NAME_LEN - 1);
 	_get_event_code(&dft_evt, cpu_type);
