@@ -1,5 +1,5 @@
 /**
- * @file pe_profiling/operf_counter.cpp
+ * @file libperf_events/operf_counter.cpp
  * C++ class implementation that abstracts the user-to-kernel interface
  * for using Linux Performance Events Subsystem.
  *
@@ -11,7 +11,7 @@
  * (C) Copyright IBM Corp. 2011
  *
  * Modified by Maynard Johnson <maynardj@us.ibm.com>
- * (C) Copyright IBM Corporation 2012
+ * (C) Copyright IBM Corporation 2012, 2014
  *
 */
 
@@ -31,6 +31,7 @@
 #include "operf_process_info.h"
 #include "op_libiberty.h"
 #include "operf_stats.h"
+#include "op_pe_utils.h"
 
 
 using namespace std;
@@ -49,6 +50,7 @@ extern bool throttled;
 extern size_t mmap_size;
 extern size_t pg_sz;
 extern bool use_cpu_minus_one;
+extern bool track_new_forks;
 
 namespace {
 
@@ -58,6 +60,15 @@ static const char *__op_magic = "OPFILE";
 
 #define OP_MAGIC	(*(u64 *)__op_magic)
 
+static bool _print_pp_progress(int fd)
+{
+	int msg;
+	if (read(fd, &msg, sizeof(msg)) > 0)
+		return true;
+	else
+		return false;
+}
+
 /* This function for reading an event from the sample data pipe must
  * be robust enough to handle the situation where the operf_record process
  * writes an event record to the pipe in multiple chunks.
@@ -96,7 +107,7 @@ again:
 		// Implies pipe has been closed on the write end, so return -1 to quit reading
 		rc = OP_PIPE_CLOSED;
 		goto out;
-	} else if (num_read != read_size) {
+	} else if (num_read != (ssize_t)read_size) {
 		header += num_read;
 		read_size -= num_read;
 		goto again;
@@ -136,10 +147,10 @@ again2:
 		// Implies pipe has been closed on the write end, so return -1 to quit reading
 		rc = OP_PIPE_CLOSED;
 		goto out;
-	} else if (num_read != read_size) {
+	} else if (num_read != (ssize_t)read_size) {
 		evt += num_read;
 		read_size -= num_read;
-		goto again;
+		goto again2;
 	}
 
 out:
@@ -206,14 +217,19 @@ operf_counter::operf_counter(operf_event_t & evt,  bool enable_on_exec, bool do_
 		attr.sample_type |= PERF_SAMPLE_CALLCHAIN;
 	if (separate_cpu)
 		attr.sample_type |= PERF_SAMPLE_CPU;
+
+#ifdef __s390__
+	attr.type = PERF_TYPE_HARDWARE;
+#else
 	attr.type = PERF_TYPE_RAW;
+#endif
 #if ((defined(__i386__) || defined(__x86_64__)) && (HAVE_PERF_PRECISE_IP))
 	if (evt.evt_code & EXTRA_PEBS) {
 		attr.precise_ip = 2;
 		evt.evt_code ^= EXTRA_PEBS;
 	}
 #endif
-	
+	attr.exclude_hv = evt.no_hv;
 	attr.config = evt.evt_code;
 	attr.sample_period = evt.count;
 	attr.inherit = inherit ? 1 : 0;
@@ -221,7 +237,6 @@ operf_counter::operf_counter(operf_event_t & evt,  bool enable_on_exec, bool do_
 	attr.disabled  = 1;
 	attr.exclude_idle = 0;
 	attr.exclude_kernel = evt.no_kernel;
-	attr.exclude_hv = evt.no_hv;
 	attr.read_format = PERF_FORMAT_ID;
 	event_name = evt.name;
 	fd = id = -1;
@@ -232,7 +247,7 @@ operf_counter::~operf_counter() {
 }
 
 
-int operf_counter::perf_event_open(pid_t pid, int cpu, operf_record * rec)
+int operf_counter::perf_event_open(pid_t pid, int cpu, operf_record * rec, bool print_error)
 {
 	struct {
 		u64 count;
@@ -248,16 +263,21 @@ int operf_counter::perf_event_open(pid_t pid, int cpu, operf_record * rec)
 		int ret = -1;
 		cverb << vrecord << "perf_event_open failed: " << strerror(errno) << endl;
 		if (errno == EBUSY) {
-			cerr << "The performance monitoring hardware reports EBUSY. Is another profiling tool in use?" << endl
-			     << "On some architectures, tools such as oprofile and perf being used in system-wide "
-			     << "mode can cause this problem." << endl;
+			if (print_error) {
+				cerr << "The performance monitoring hardware reports EBUSY. Is another profiling tool in use?" << endl
+				     << "On some architectures, tools such as oprofile and perf being used in system-wide "
+				     << "mode can cause this problem." << endl;
+			}
 			ret = OP_PERF_HANDLED_ERROR;
 		} else if (errno == ESRCH) {
-			cerr << "!!!! No samples collected !!!" << endl;
-			cerr << "The target program/command ended before profiling was started." << endl;
+			if (print_error) {
+				cerr << "!!!! No samples collected !!!" << endl;
+				cerr << "The target program/command ended before profiling was started." << endl;
+			}
 			ret = OP_PERF_HANDLED_ERROR;
 		} else {
-			cerr << "perf_event_open failed with " << strerror(errno) << endl;
+			if (print_error)
+				cerr << "perf_event_open failed with " << strerror(errno) << endl;
 		}
 		return ret;
 	}
@@ -282,7 +302,7 @@ operf_record::~operf_record()
 
 	if (poll_data)
 		delete[] poll_data;
-	for (int i = 0; i < samples_array.size(); i++) {
+	for (size_t i = 0; i < samples_array.size(); i++) {
 		struct mmap_data *md = &samples_array[i];
 		munmap(md->base, (num_mmap_pages + 1) * pagesize);
 	}
@@ -298,9 +318,9 @@ operf_record::~operf_record()
 
 operf_record::operf_record(int out_fd, bool sys_wide, pid_t the_pid, bool pid_running,
                            vector<operf_event_t> & events, vmlinux_info_t vi, bool do_cg,
-                           bool separate_by_cpu, bool out_fd_is_file)
+                           bool separate_by_cpu, bool out_fd_is_file,
+                           int _convert_read_pipe, int _convert_write_pipe)
 {
-	int flags = O_CREAT|O_RDWR|O_TRUNC;
 	struct sigaction sa;
 	sigset_t ss;
 	vmlinux_file = vi.image_name;
@@ -317,6 +337,8 @@ operf_record::operf_record(int out_fd, bool sys_wide, pid_t the_pid, bool pid_ru
 	valid = false;
 	poll_data = NULL;
 	output_fd = out_fd;
+	read_comm_pipe = _convert_read_pipe;
+	write_comm_pipe = _convert_write_pipe;
 	write_to_file = out_fd_is_file;
 	opHeader.data_size = 0;
 	num_cpus = -1;
@@ -539,7 +561,7 @@ int operf_record::prepareToRecord(void)
 			     << num_cpus << " x " << evts.size() << ")." << endl;
 			return -1;
 		}
-		for (unsigned int cpu = 0; cpu < num_cpus; cpu++) {
+		for (int cpu = 0; cpu < num_cpus; cpu++) {
 			int fd_for_set_output = perfCounters[op_ctr_idx].get_fd();
 			for (unsigned event = 0; event < evts.size(); event++) {
 				int fd = perfCounters[op_ctr_idx].get_fd();
@@ -618,8 +640,7 @@ void operf_record::setup()
 		goto error;
 	}
 	memset(cpus_online, 0, sizeof(cpus_online));
-	fgets(cpus_online, sizeof(cpus_online), online_cpus);
-	if (!cpus_online[0]) {
+	if (fgets(cpus_online, sizeof(cpus_online), online_cpus) == NULL) {
 		fclose(online_cpus);
 		err_msg = "Internal Error: Number of online cpus cannot be determined.";
 		rc = -1;
@@ -639,13 +660,12 @@ void operf_record::setup()
 
 	for (int cpu = 0; cpu < num_cpus; cpu++) {
 		int real_cpu;
-		int mmap_fd;
 		if (use_cpu_minus_one) {
 			real_cpu = -1;
 		} else if (all_cpus_avail) {
 			real_cpu = cpu;
 		} else {
-			real_cpu = op_get_next_online_cpu(dir, entry);
+			real_cpu = op_pe_utils::op_get_next_online_cpu(dir, entry);
 			if (real_cpu < 0) {
 				err_msg = "Internal Error: Number of online cpus cannot be determined.";
 				rc = -1;
@@ -659,6 +679,7 @@ void operf_record::setup()
 		 * "inherit" on the perf_event_attr we pass to perf_event_open when cpu is '-1'.
 		 */
 		bool inherit = !profile_process_group;
+		std::map<u32, struct comm_event>::iterator proc_it = procs.begin();
 		for (unsigned proc_idx = 0; proc_idx < num_procs; proc_idx++) {
 			for (unsigned event = 0; event < evts.size(); event++) {
 				/* For a parent process, comm.tid==comm.pid, but for child
@@ -666,14 +687,17 @@ void operf_record::setup()
 				 * we must use comm.tid for the perf_event_open call.  So
 				 * we can use comm.tid for all cases.
 				 */
-				pid_t pid_for_open = profile_process_group ? procs[proc_idx].tid
-				                                           : pid_to_profile;
+				pid_t pid_for_open;
+				if (profile_process_group)
+					pid_for_open = proc_it++->second.tid;
+				else
+					pid_for_open = pid_to_profile;
 				operf_counter op_ctr(operf_counter(evts[event],
 				                                   (!pid_started && !system_wide),
 				                                   callgraph, separate_cpu,
 				                                   inherit, event));
 				if ((rc = op_ctr.perf_event_open(pid_for_open,
-				                                 real_cpu, this)) < 0) {
+				                                 real_cpu, this, true)) < 0) {
 					err_msg = "Internal Error.  Perf event setup failed.";
 					goto error;
 				}
@@ -702,7 +726,7 @@ void operf_record::setup()
 error:
 	delete[] poll_data;
 	poll_data = NULL;
-	for (int i = 0; i < samples_array.size(); i++) {
+	for (size_t i = 0; i < samples_array.size(); i++) {
 		struct mmap_data *md = &samples_array[i];
 		munmap(md->base, (num_mmap_pages + 1) * pagesize);
 	}
@@ -718,36 +742,109 @@ void operf_record::record_process_info(void)
 {
 	map<unsigned int, unsigned int> pids_mapped;
 	pid_t last_tgid = -1;
-	for (unsigned int proc_idx = 0; proc_idx < procs.size(); proc_idx++)
+	std::map<u32, struct comm_event>::iterator proc_it = procs.begin();
+	for (unsigned int proc_idx = 0; proc_idx < procs.size(); proc_idx++, proc_it++)
 	{
-		int num = OP_perf_utils::op_write_output(output_fd, &procs[proc_idx],
-		                                         procs[proc_idx].header.size);
+		struct comm_event ce = proc_it->second;
+		int num = OP_perf_utils::op_write_output(output_fd, &ce, ce.header.size);
 		add_to_total(num);
 		if (cverb << vrecord)
-			cout << "Created COMM event for " << procs[proc_idx].comm << endl;
+			cout << "Created COMM event for " << ce.comm << endl;
 
-		if ((procs[proc_idx].pid == last_tgid) ||
-				(pids_mapped.find(procs[proc_idx].pid) != pids_mapped.end()))
+		if (((pid_t)(ce.pid) == last_tgid) ||
+				(pids_mapped.find(ce.pid) != pids_mapped.end()))
 			continue;
-		OP_perf_utils::op_record_process_exec_mmaps(procs[proc_idx].tid,
-		                                            procs[proc_idx].pid,
+		OP_perf_utils::op_record_process_exec_mmaps(ce.tid,
+		                                            ce.pid,
 		                                            output_fd, this);
-		pids_mapped[procs[proc_idx].pid] = last_tgid = procs[proc_idx].pid;
+		pids_mapped[ce.pid] = last_tgid = ce.pid;
 	}
 }
 
+int operf_record::_start_recoding_new_thread(pid_t id)
+{
+	string err_msg;
+	int num_mmaps, rc, fd_for_set_output = -1;
+	struct comm_event ce;
+	u64 sample_id;
+	struct pollfd * old_polldata = poll_data;
+
+	num_mmaps = sizeof(poll_data)/sizeof(poll_data[0]);
+	num_mmaps++;
+	poll_data = new struct pollfd [num_mmaps];
+	// Copy only the existing pollfd objects from the array.  The new pollfd will
+	// be filled in via the call to _prepare_to_record_one_fd.
+	for (int i = 0; i < num_mmaps - 1; i++)
+		poll_data[i] = old_polldata[i];
+	delete[] old_polldata;
+	// Make a pseudo comm_event object.  At this point, the
+	// only field we need to set is tid.
+	memset(&ce, 0, sizeof(ce));
+	ce.tid = id;
+	add_process(ce);
+
+	for (unsigned event = 0; event < evts.size(); event++) {
+		operf_counter op_ctr(operf_counter(evts[event],
+		                                   (!pid_started && !system_wide),
+		                                   callgraph, separate_cpu,
+		                                   false, event));
+		if (op_ctr.perf_event_open(id, -1, this, false) < 0) {
+			sample_id = OP_PERF_NO_SAMPLE_ID;
+			// Send special value to convert process to indicate failure
+			ssize_t len = write(write_comm_pipe, &sample_id, sizeof(sample_id));
+			if (len < 0)
+				perror("Internal error on convert write_comm_pipe");
+			return -1;
+		}
+		perfCounters.push_back(op_ctr);
+		int fd = op_ctr.get_fd();
+		if (event == 0) {
+			rc = _prepare_to_record_one_fd(num_mmaps - 1, fd);
+			fd_for_set_output = fd;
+		} else {
+			if ((rc = ioctl(fd, PERF_EVENT_IOC_SET_OUTPUT,
+			                fd_for_set_output)) < 0)
+				perror("_start_recoding_new_thread: ioctl #1 failed");
+		}
+
+		if (rc < 0)
+			return rc;
+
+		if ((rc = ioctl(fd, PERF_EVENT_IOC_ENABLE)) < 0) {
+			perror("_start_recoding_new_thread: ioctl #2 failed");
+			return rc;
+		}
+
+		sample_id = opHeader.h_attrs[event].ids.back();
+		ssize_t len = write(write_comm_pipe, &sample_id, sizeof(sample_id));
+		if (len < 0)
+			perror("Internal error on convert write_comm_pipe");
+		else if (len != sizeof(sample_id))
+			cerr << "Incomplete write convert to write_comm_pipe" << endl;
+		else
+			cverb << vrecord << "Sent sample_id " << sample_id << " to convert process" << endl;
+	}
+
+	return 0;
+}
+
+
 void operf_record::recordPerfData(void)
 {
 	bool disabled = false;
 	if (pid_started || system_wide)
 		record_process_info();
+	else
+		op_get_vsyscall_mapping(pid_to_profile, output_fd, this);
 
 	op_record_kernel_info(vmlinux_file, kernel_start, kernel_end, output_fd, this);
 	cerr << "operf: Profiler started" << endl;
 	while (1) {
 		int prev = sample_reads;
+		pid_t pi;
+		ssize_t len;
 
-		for (int i = 0; i < samples_array.size(); i++) {
+		for (size_t i = 0; i < samples_array.size(); i++) {
 			if (samples_array[i].base)
 				op_get_kernel_event_data(&samples_array[i], this);
 		}
@@ -757,6 +854,20 @@ void operf_record::recordPerfData(void)
 		if (prev == sample_reads) {
 			(void)poll(poll_data, poll_count, -1);
 		}
+		if (!quit && track_new_forks && procs.size() > 1) {
+			len = read(read_comm_pipe, &pi, sizeof(pi));
+
+			if (len < 0 && errno != EAGAIN) {
+				cverb << vrecord << "Non-fatal error: read_comm_pipe returned too few bytes" << endl;
+			} else if (len == sizeof(pi) && (procs.find(pi) == procs.end())) {
+				// Start profiling this new thread
+				cverb << vrecord << "Start recording for new thread " << pi << endl;
+				// Don't treat as fatal error if it doesn't work
+				if (_start_recoding_new_thread(pi) < 0)
+					cerr << "Unable to collect samples for forked process " << pi
+					     << ". Process may have ended before recording could be started." << endl;
+			}
+		}
 
 		if (quit) {
 			for (unsigned int i = 0; i < perfCounters.size(); i++)
@@ -770,12 +881,15 @@ void operf_record::recordPerfData(void)
 }
 
 void operf_read::init(int sample_data_pipe_fd, string input_filename, string samples_loc, op_cpu cputype,
-                      vector<operf_event_t> & events, bool systemwide)
+                      bool systemwide, int _record_write_pipe, int _record_read_pipe,
+                      int _post_profiling_pipe)
 {
 	sample_data_fd = sample_data_pipe_fd;
+	read_comm_pipe = _record_read_pipe;
+	write_comm_pipe = _record_write_pipe;
+	post_profiling_pipe = _post_profiling_pipe;
 	inputFname = input_filename;
 	sampledir = samples_loc;
-	evts = events;
 	cpu_type = cputype;
 	syswide = systemwide;
 }
@@ -785,6 +899,11 @@ operf_read::~operf_read()
 	evts.clear();
 }
 
+void operf_read::add_sample_id_to_opHeader(u64 sample_id)
+{
+	for (unsigned int i = 0; i < evts.size(); i++)
+		opHeader.h_attrs[i].ids.push_back(sample_id);
+}
 
 int operf_read::_read_header_info_with_ifstream(void)
 {
@@ -973,7 +1092,12 @@ unsigned int operf_read::convertPerfData(void)
 	unsigned int num_bytes = 0;
 	struct mmap_info info;
 	bool error = false;
-	event_t * event;
+	event_t * event = NULL;
+
+	if (fcntl(post_profiling_pipe, F_SETFL, O_NONBLOCK) < 0) {
+		cerr << "Error: fcntl failed with errno:\n\t" << strerror(errno) << endl;
+		throw runtime_error("Error: Unable to set post_profiling_pipe to non blocking");
+	}
 
 	if (!inputFname.empty()) {
 		info.file_data_offset = opHeader.data_offset;
@@ -1009,8 +1133,7 @@ unsigned int operf_read::convertPerfData(void)
 	int num_recs = 0;
 	struct perf_event_header last_header;
 	bool print_progress = !inputFname.empty() && syswide;
-	if (print_progress)
-		cerr << "Converting profile data to OProfile format" << endl;
+	bool printed_progress_msg = false;
 	while (1) {
 		streamsize rec_size = 0;
 		if (!inputFname.empty()) {
@@ -1031,10 +1154,14 @@ unsigned int operf_read::convertPerfData(void)
 		}
 		num_bytes += rec_size;
 		num_recs++;
-		if ((num_recs % 1000000 == 0) && print_progress)
+		if ((num_recs % 1000000 == 0) && (print_progress || _print_pp_progress(post_profiling_pipe))) {
+			if (!printed_progress_msg) {
+				cerr << "\nConverting profile data to OProfile format " << endl;
+				printed_progress_msg = true;
+			}
 			cerr << ".";
+		}
 	}
-
 	if (unlikely(error)) {
 		if (!inputFname.empty()) {
 			cerr << "ERROR: operf_read::convertPerfData quitting. Bad data read from file." << endl;
@@ -1052,7 +1179,7 @@ unsigned int operf_read::convertPerfData(void)
 	if (!error)
 		op_reprocess_unresolved_events(opHeader.h_attrs[0].attr.sample_type, print_progress);
 
-	if (print_progress)
+	if (printed_progress_msg)
 		cerr << endl;
 
 	op_release_resources();
