@@ -10,7 +10,7 @@
  * (C) Copyright IBM Corp. 2011
  *
  * Modified by Maynard Johnson <maynardj@us.ibm.com>
- * (C) Copyright IBM Corporation 2012, 2013
+ * (C) Copyright IBM Corporation 2012, 2013, 2014
  *
  */
 
@@ -36,11 +36,10 @@
 #include "op_fileio.h"
 #include "op_libiberty.h"
 #include "operf_stats.h"
+#include "utility.h"
 
 
-extern verbose vmisc;
 extern volatile bool quit;
-extern volatile bool read_quit;
 extern operf_read operfRead;
 extern int sample_reads;
 extern unsigned int pagesize;
@@ -49,6 +48,7 @@ extern pid_t app_PID;
 extern verbose vrecord;
 extern verbose vconvert;
 extern void __set_event_throttled(int index);
+extern bool track_new_forks;
 
 using namespace std;
 
@@ -65,161 +65,6 @@ static list<event_t *> unresolved_events;
 static struct operf_transient trans;
 static bool sfile_init_done;
 
-/* Some architectures (e.g., ppc64) do not use the same event value (code) for oprofile
- * and for perf_events.  The operf-record process requires event values that perf_events
- * understands, but the operf-read process requires oprofile event values.  The purpose of
- * the following method is to map the operf-record event value to a value that
- * opreport can understand.
- */
-#if PPC64_ARCH
-extern op_cpu cpu_type;
-#define NIL_CODE ~0U
-
-#if HAVE_LIBPFM3
-static bool _get_codes_for_match(unsigned int pfm_idx, const char name[],
-                                 vector<operf_event_t> * evt_vec)
-{
-	unsigned int num_events = evt_vec->size();
-	int tmp_code, ret;
-	char evt_name[OP_MAX_EVT_NAME_LEN];
-	unsigned int events_converted = 0;
-	for (unsigned int i = 0; i < num_events; i++) {
-		operf_event_t event = (*evt_vec)[i];
-		if (event.evt_code != NIL_CODE) {
-			events_converted++;
-			continue;
-		}
-		memset(evt_name, 0, OP_MAX_EVT_NAME_LEN);
-		if (!strcmp(event.name, "CYCLES")) {
-			strcpy(evt_name ,"PM_CYC") ;
-		} else if (strstr(event.name, "_GRP")) {
-			string str = event.name;
-			strncpy(evt_name, event.name, str.rfind("_GRP"));
-		} else {
-			strncpy(evt_name, event.name, strlen(event.name));
-		}
-		if (strncmp(name, evt_name, OP_MAX_EVT_NAME_LEN))
-			continue;
-		ret = pfm_get_event_code(pfm_idx, &tmp_code);
-		if (ret != PFMLIB_SUCCESS) {
-			string evt_name_str = event.name;
-			string msg = "libpfm cannot find event code for " + evt_name_str +
-					"; cannot continue";
-			throw runtime_error(msg);
-		}
-		event.evt_code = tmp_code;
-		(*evt_vec)[i] = event;
-		events_converted++;
-		cverb << vrecord << "Successfully converted " << event.name << " to perf_event code "
-		      << hex << tmp_code << endl;
-	}
-	return (events_converted == num_events);
-}
-#else
-static bool _op_get_event_codes(vector<operf_event_t> * evt_vec)
-{
-	int ret, i;
-	unsigned int num_events = evt_vec->size();
-	char evt_name[OP_MAX_EVT_NAME_LEN];
-	unsigned int events_converted = 0;
-	uint64_t code[1];
-
-	typedef struct {
-		uint64_t    *codes;
-		char        **fstr;
-		size_t      size;
-		int         count;
-		int         idx;
-	} pfm_raw_pmu_encode_t;
-
-	pfm_raw_pmu_encode_t raw;
-	raw.codes = code;
-	raw.count = 1;
-	raw.fstr = NULL;
-
-	if (pfm_initialize() != PFM_SUCCESS)
-		throw runtime_error("Unable to initialize libpfm; cannot continue");
-
-	for (unsigned int i = 0; i < num_events; i++) {
-		operf_event_t event = (*evt_vec)[i];
-		if (event.evt_code != NIL_CODE) {
-			events_converted++;
-			continue;
-		}
-		memset(evt_name, 0, OP_MAX_EVT_NAME_LEN);
-		if (!strcmp(event.name, "CYCLES")) {
-			strcpy(evt_name ,"PM_CYC") ;
-		} else if (strstr(event.name, "_GRP")) {
-			string str = event.name;
-			strncpy(evt_name, event.name, str.rfind("_GRP"));
-		} else {
-			strncpy(evt_name, event.name, strlen(event.name));
-		}
-
-		memset(&raw, 0, sizeof(raw));
-		ret = pfm_get_os_event_encoding(evt_name, PFM_PLM3, PFM_OS_NONE, &raw);
-		if (ret != PFM_SUCCESS) {
-			string evt_name_str = event.name;
-			string msg = "libpfm cannot find event code for " + evt_name_str +
-					"; cannot continue";
-			throw runtime_error(msg);
-		}
-
-		event.evt_code = raw.codes[0];
-		(*evt_vec)[i] = event;
-		events_converted++;
-		cverb << vrecord << "Successfully converted " << event.name << " to perf_event code "
-		      << hex << event.evt_code << endl;
-	}
-	return (events_converted == num_events);
-}
-#endif
-
-bool OP_perf_utils::op_convert_event_vals(vector<operf_event_t> * evt_vec)
-{
-	unsigned int i, count;
-	char name[256];
-	int ret;
-	for (unsigned int i = 0; i < evt_vec->size(); i++) {
-		operf_event_t event = (*evt_vec)[i];
-		if (cpu_type == CPU_PPC64_POWER7) {
-			if (!strncmp(event.name, "PM_RUN_CYC", strlen("PM_RUN_CYC"))) {
-				event.evt_code = 0x600f4;
-			} else if (!strncmp(event.name, "PM_RUN_INST_CMPL", strlen("PM_RUN_INST_CMPL"))) {
-				event.evt_code = 0x500fa;
-			} else {
-				event.evt_code = NIL_CODE;
-			}
-		} else {
-			event.evt_code = NIL_CODE;
-		}
-		(*evt_vec)[i] = event;
-	}
-
-#if HAVE_LIBPFM3
-	if (pfm_initialize() != PFMLIB_SUCCESS)
-		throw runtime_error("Unable to initialize libpfm; cannot continue");
-
-	ret = pfm_get_num_events(&count);
-	if (ret != PFMLIB_SUCCESS)
-		throw runtime_error("Unable to use libpfm to obtain event code; cannot continue");
-	for(i =0 ; i < count; i++)
-	{
-		ret = pfm_get_event_name(i, name, 256);
-		if (ret != PFMLIB_SUCCESS)
-			continue;
-		if (_get_codes_for_match(i, name, evt_vec))
-			break;
-	}
-	return (i != count);
-#else
-	return _op_get_event_codes(evt_vec);
-#endif
-}
-
-#endif // PPC64_ARCH
-
-
 static inline void update_trans_last(struct operf_transient * trans)
 {
 	trans->last = trans->current;
@@ -243,6 +88,8 @@ static void __handle_fork_event(event_t * event)
 	operf_process_info * parent = NULL;
 	operf_process_info * forked_proc = NULL;
 
+	// First, see if we already have a proc_info object for the parent process
+	// that did the fork
 	it = process_map.find(event->fork.ppid);
 	if (it != process_map.end()) {
 		parent = it->second;
@@ -257,14 +104,42 @@ static void __handle_fork_event(event_t * event)
 		process_map[event->fork.ppid] = parent;
 	}
 
-	 /* If the forked process's pid is the same as the parent's, we simply ignore the FORK
-	 * event. This is because operf_process_info objects are stored in the map collection
-	 * by pid, meaning that the forked process and its parent reference the same
+	/* If the user requested to profile by "--pid", then we must notify the
+	 * recording process whenever we see a fork event. If the record process
+	 * isn't already recording samples for this thread/process, it will start
+	 * recording now.
+	 */
+	if (track_new_forks) {
+		if (cverb << vconvert)
+			cout << "Inform record process of new pid/tid "
+			     << event->fork.pid << "/" << event->fork.tid << endl;
+		pid_t id = (event->fork.pid == event->fork.ppid) ? event->fork.tid :
+				event->fork.pid;
+		ssize_t len = write(operfRead.get_write_comm_pipe(), &id, sizeof(id));
+		if (len < 0)
+			perror("Internal error on record write_comm_pipe");
+		else if (len != sizeof(id))
+			cerr << "Incomplete write to record write_comm_pipe" << endl;
+		u64 sample_id;
+		// get sample id from recording process
+		len = read(operfRead.get_read_comm_pipe(), &sample_id, sizeof(sample_id));
+		if (sample_id == OP_PERF_NO_SAMPLE_ID) {
+			cverb << vconvert << "convert: No sample_id from record process" << endl;
+		} else {
+			cverb << vconvert << "Add sample_id " << sample_id << " to opHeader" << endl;
+			operfRead.add_sample_id_to_opHeader(sample_id);
+		}
+	}
+
+	/* If the forked process's pid is the same as the parent's, we simply ignore
+	 * the FORK event. This is because operf_process_info objects are stored in the map
+	 * collection by pid, meaning that the forked process and its parent reference the same
 	 * operf_process_info object.
 	 */
 	if (event->fork.pid == event->fork.ppid)
 		return;
 
+	// Now try to find a proc_info for the forked process itself.
 	it = process_map.find(event->fork.pid);
 	if (it == process_map.end()) {
 		forked_proc = new operf_process_info(event->fork.pid, NULL, false, false);
@@ -325,7 +200,7 @@ static void __handle_comm_event(event_t * event)
 		 */
 		const char * appname_arg;
 		bool is_complete_appname;
-		if (app_name && (app_PID == event->comm.pid)) {
+		if (app_name && (app_PID == (pid_t) event->comm.pid)) {
 			appname_arg = app_name;
 			is_complete_appname = true;
 		} else {
@@ -475,7 +350,7 @@ static void __handle_mmap_event(event_t * event)
 			 */
 			const char * appname_arg;
 			bool is_complete_appname;
-			if (app_name && (app_PID == event->mmap.pid)) {
+			if (app_name && (app_PID == (pid_t)event->mmap.pid)) {
 				appname_arg = app_name;
 				is_complete_appname = true;
 			} else {
@@ -555,7 +430,7 @@ static struct operf_transient * __get_operf_trans(struct sample_data * data, boo
 			 */
 		}
 	} else {
-		op_mmap = proc->find_mapping_for_sample(data->ip);
+		op_mmap = proc->find_mapping_for_sample(data->ip, hypervisor_domain);
 		if (op_mmap && op_mmap->is_hypervisor && !hypervisor_domain) {
 			cverb << vconvert << "Invalid sample: Address falls within hypervisor address range, but is not a hypervisor domain sample." << endl;
 			operf_stats[OPERF_INVALID_CTX]++;
@@ -587,13 +462,15 @@ static struct operf_transient * __get_operf_trans(struct sample_data * data, boo
 		trans.sample_id = data->id;
 		retval = &trans;
 	} else {
-		if ((cverb << vconvert) && !first_time_processing) {
-			string domain = trans.in_kernel ? "kernel" : "userspace";
-			ostringstream message;
-			message << "Discarding " << domain << " sample for process " << data->pid
-			        << " where no appropriate mapping was found. (pc=0x"
-			        << hex << data->ip <<")" << endl;
-			cout << message.str();
+		if (!first_time_processing) {
+			if (cverb << vconvert) {
+				string domain = trans.in_kernel ? "kernel" : "userspace";
+				ostringstream message;
+				message << "Discarding " << domain << " sample for process " << data->pid
+						<< " where no appropriate mapping was found. (pc=0x"
+						<< hex << data->ip <<")" << endl;
+				cout << message.str();
+			}
 			operf_stats[OPERF_LOST_NO_MAPPING]++;
 		}
 		retval = NULL;
@@ -605,11 +482,12 @@ out:
 static void __handle_callchain(u64 * array, struct sample_data * data)
 {
 	bool in_kernel = false;
+	u64 sampled_addr = data->ip;
 	data->callchain = (struct ip_callchain *) array;
 	if (data->callchain->nr) {
 		if (cverb << vconvert)
 			cout << "Processing callchain" << endl;
-		for (int i = 0; i < data->callchain->nr; i++) {
+		for (u64 i = 0; i < data->callchain->nr; i++) {
 			data->ip = data->callchain->ips[i];
 			if (data->ip >= PERF_CONTEXT_MAX) {
 				switch (data->ip) {
@@ -626,6 +504,8 @@ static void __handle_callchain(u64 * array, struct sample_data * data)
 					default:
 						break;
 				}
+				if (i == 0 && (data->callchain->ips[i+1]==sampled_addr))
+					i++;
 				continue;
 			}
 			if (data->ip && __get_operf_trans(data, false, in_kernel)) {
@@ -634,13 +514,14 @@ static void __handle_callchain(u64 * array, struct sample_data * data)
 					update_trans_last(&trans);
 				}
 			} else {
-				if (data->ip)
+				if (data->ip && !first_time_processing)
 					operf_stats[OPERF_BT_LOST_NO_MAPPING]++;
 			}
 		}
 	}
 }
 
+#if PPC64_ARCH
 static void __map_hypervisor_sample(u64 ip, u32 pid)
 {
 	operf_process_info * proc;
@@ -657,7 +538,7 @@ static void __map_hypervisor_sample(u64 ip, u32 pid)
 		 */
 		const char * appname_arg;
 		bool is_complete_appname;
-		if (app_name && (app_PID == pid)) {
+		if (app_name && (app_PID == (pid_t)pid)) {
 			appname_arg = app_name;
 			is_complete_appname = true;
 		} else {
@@ -677,8 +558,9 @@ static void __map_hypervisor_sample(u64 ip, u32 pid)
 	}
 	proc->process_hypervisor_mapping(ip);
 }
+#endif
 
-static int __handle_throttle_event(event_t * event, u64 sample_type)
+static int __handle_throttle_event(event_t * event)
 {
 	int rc = 0;
 	trans.event = operfRead.get_eventnum_by_perf_event_id(event->throttle.id);
@@ -695,7 +577,6 @@ static int __handle_sample_event(event_t * event, u64 sample_type)
 	bool found_trans = false;
 	bool in_kernel;
 	int rc = 0;
-	const struct operf_mmap * op_mmap = NULL;
 	bool hypervisor = (event->header.misc == PERF_RECORD_MISC_HYPERVISOR);
 	u64 *array = event->sample.array;
 
@@ -842,6 +723,10 @@ static int __handle_sample_event(event_t * event, u64 sample_type)
 			cout << "Deferring processing of hypervisor sample." << endl;
 		goto out;
 	}
+	// This sample is for a different event than the last sample
+	if (data.id != trans.sample_id)
+		goto find_trans;
+
 	/* Check for the common case first -- i.e., where the current sample is from
 	 * the same context as the previous sample.  For the "no-vmlinux" case, start_addr
 	 * and end_addr will be zero, so need to make sure we detect that.
@@ -869,6 +754,7 @@ static int __handle_sample_event(event_t * event, u64 sample_type)
 		found_trans = true;
 	}
 
+find_trans:
 	if (!found_trans && __get_operf_trans(&data, hypervisor, in_kernel)) {
 		trans.current = operf_sfile_find(&trans);
 		found_trans = true;
@@ -938,7 +824,7 @@ int OP_perf_utils::op_write_event(event_t * event, u64 sample_type)
 		__handle_fork_event(event);
 		return 0;
 	case PERF_RECORD_THROTTLE:
-		return __handle_throttle_event(event, sample_type);
+		return __handle_throttle_event(event);
 	case PERF_RECORD_LOST:
 		operf_stats[OPERF_RECORD_LOST_SAMPLE] += event->lost.lost;
 		return 0;
@@ -1108,6 +994,72 @@ int OP_perf_utils::op_write_output(int output, void *buf, size_t size)
 	return sum;
 }
 
+/* On certain architectures and older kernels (3.0 and older, I think), a static mapping
+ * was placed into every process's memory map to provide vsyscall functionality.  The
+ * mapping is labeled '[vsyscall]'.  For some reason (which I don't care to investigate,
+ * since vsyscall is now obsolete), the kernel's perf_events subsystem does not send a
+ * PERF_RECORD_MMAP message for this mapping.  The function below is used to synthesize
+ * such a message so that samples taken in the vsyscall memory range can be correctly
+ * attributed.
+ */
+void OP_perf_utils::op_get_vsyscall_mapping(pid_t tgid, int output_fd, operf_record * pr)
+{
+	char fname[PATH_MAX];
+	FILE *fp;
+	char line_buffer[BUFSIZ];
+	char perms[5], pathname[PATH_MAX], dev[16];
+	unsigned long long start_addr, end_addr, offset;
+	u_int32_t inode;
+	struct mmap_event mmap;
+	size_t size;
+
+	memset(pathname, '\0', sizeof(pathname));
+	memset(&mmap, 0, sizeof(mmap));
+
+	snprintf(fname, sizeof(fname), "/proc/%d/maps", tgid);
+
+	fp = fopen(fname, "r");
+	if (fp == NULL) {
+		// Process must have exited already or invalid pid.
+		cverb << vrecord << "couldn't open " << fname << endl;
+		return;
+	}
+
+	while (1) {
+		mmap.pgoff = 0;
+		mmap.header.type = PERF_RECORD_MMAP;
+		mmap.header.misc = PERF_RECORD_MISC_USER;
+
+		if (fgets(line_buffer, sizeof(line_buffer), fp) == NULL)
+			break;
+
+		sscanf(line_buffer, "%llx-%llx %s %llx %s %d %s",
+				&start_addr, &end_addr, perms, &offset, dev, &inode, pathname);
+		if (perms[2] == 'x') {
+			char * imagename;
+			if ((imagename = strstr(pathname, "[vsyscall]")) == NULL)
+				continue;
+
+			size = strlen(imagename) + 1;
+			strcpy(mmap.filename, imagename);
+			size = align_64bit(size);
+			mmap.start = start_addr;
+			mmap.len = end_addr - mmap.start;
+			mmap.pid = tgid;
+			mmap.tid = tgid;
+			mmap.header.size = (sizeof(mmap) -
+					(sizeof(mmap.filename) - size));
+			int num = OP_perf_utils::op_write_output(output_fd, &mmap, mmap.header.size);
+			if (cverb << vrecord)
+				cout << "Created MMAP event for " << imagename << endl;
+			pr->add_to_total(num);
+			break;
+		}
+	}
+
+	fclose(fp);
+	return;
+}
 
 void OP_perf_utils::op_record_process_exec_mmaps(pid_t pid, pid_t tgid, int output_fd, operf_record * pr)
 {
@@ -1150,6 +1102,9 @@ void OP_perf_utils::op_record_process_exec_mmaps(pid_t pid, pid_t tgid, int outp
 			if (imagename == NULL)
 				imagename = strstr(pathname, "[vdso]");
 
+			if (imagename == NULL)
+				imagename = strstr(pathname, "[vsyscall]");
+
 			if ((imagename == NULL) && !strstr(pathname, "["))
 				imagename = (char *)anon_mem;
 
@@ -1283,9 +1238,6 @@ int OP_perf_utils::op_get_process_info(bool system_wide, pid_t pid, operf_record
 	if (!system_wide) {
 		ret = _get_one_process_info(system_wide, pid, pr);
 	} else {
-		char buff[BUFSIZ];
-		pid_t tgid = 0;
-		size_t size = 0;
 		DIR *pids;
 		struct dirent dirent, *next;
 
@@ -1298,11 +1250,8 @@ int OP_perf_utils::op_get_process_info(bool system_wide, pid_t pid, operf_record
 		while (!readdir_r(pids, &dirent, &next) && next) {
 			char *end;
 			pid = strtol(dirent.d_name, &end, 10);
-			if (((errno == ERANGE && (pid == LONG_MAX || pid == LONG_MIN))
-					|| (errno != 0 && pid == 0)) || (end == dirent.d_name)) {
-				cverb << vmisc << "/proc entry " << dirent.d_name << " is not a PID" << endl;
+			if (*end)
 				continue;
-			}
 			if ((ret = _get_one_process_info(system_wide, pid, pr)) < 0)
 				break;
 		}
@@ -1324,7 +1273,6 @@ static void _record_module_info(int output_fd, operf_record * pr)
 	const char * fname = "/proc/modules";
 	FILE *fp;
 	char * line;
-	struct operf_kernel_image * image;
 	int module_size;
 	char ref_count[32+1];
 	int ret;
@@ -1364,6 +1312,14 @@ static void _record_module_info(int output_fd, operf_record * pr)
 			continue;
 		}
 
+		if (start_address == 0) {
+			cerr << "Unable to obtain module information. Set "
+			     << "/proc/sys/kernel/kptr_restrict to 0 to "
+			     << "collect kernel module samples." << endl;
+			fclose(fp);
+			return;
+		}
+
 		mmap.header.type = PERF_RECORD_MMAP;
 		mmap.header.misc = PERF_RECORD_MISC_KERNEL;
 		size = strlen(module_name) + 1;
@@ -1396,10 +1352,21 @@ void OP_perf_utils::op_record_kernel_info(string vmlinux_file, u64 start_addr, u
 	mmap.header.type = PERF_RECORD_MMAP;
 	mmap.header.misc = PERF_RECORD_MISC_KERNEL;
 	if (vmlinux_file.empty()) {
-		size = strlen( "no_vmlinux") + 1;
-		strncpy(mmap.filename, "no-vmlinux", size);
-		mmap.start = 0ULL;
-		mmap.len = 0ULL;
+		if ((start_addr == 0) && (end_addr == 0)) {
+			/* Did not have permission to read
+			 * /proc/kallsyms and no vmlinux file
+			 */
+			size = strlen( "no_vmlinux") + 1;
+			strncpy(mmap.filename, "no-vmlinux", size);
+			mmap.start = 0ULL;
+			mmap.len = 0ULL;
+		} else {
+			size = sizeof(KALL_SYM_FILE) + 1;
+			strncpy(mmap.filename, KALL_SYM_FILE, size);
+			mmap.start = start_addr;
+			mmap.len = end_addr - mmap.start;
+		}
+
 	} else {
 		size = vmlinux_file.length() + 1;
 		strncpy(mmap.filename, vmlinux_file.c_str(), size);
@@ -1419,7 +1386,9 @@ void OP_perf_utils::op_record_kernel_info(string vmlinux_file, u64 start_addr, u
 		cout << message.str();
 	}
 	pr->add_to_total(num);
-	_record_module_info(output_fd, pr);
+
+	if (start_addr && end_addr)
+		_record_module_info(output_fd, pr);
 }
 
 void OP_perf_utils::op_get_kernel_event_data(struct mmap_data *md, operf_record * pr)
@@ -1465,38 +1434,3 @@ void OP_perf_utils::op_get_kernel_event_data(struct mmap_data *md, operf_record
 	md->prev = old;
 	pc->data_tail = old;
 }
-
-
-int OP_perf_utils::op_get_next_online_cpu(DIR * dir, struct dirent *entry)
-{
-#define OFFLINE 0x30
-	unsigned int cpu_num;
-	char cpu_online_pathname[40];
-	int res;
-	FILE * online;
-	again:
-	do {
-		entry = readdir(dir);
-		if (!entry)
-			return -1;
-	} while (entry->d_type != DT_DIR);
-
-	res = sscanf(entry->d_name, "cpu%u", &cpu_num);
-	if (res <= 0)
-		goto again;
-
-	errno = 0;
-	snprintf(cpu_online_pathname, 40, "/sys/devices/system/cpu/cpu%u/online", cpu_num);
-	if ((online = fopen(cpu_online_pathname, "r")) == NULL) {
-		cerr << "Unable to open " << cpu_online_pathname << endl;
-		if (errno)
-			cerr << strerror(errno) << endl;
-		return -1;
-	}
-	res = fgetc(online);
-	fclose(online);
-	if (res == OFFLINE)
-		goto again;
-	else
-		return cpu_num;
-}
