@@ -27,6 +27,7 @@
 #include <string.h>
 #include <getopt.h>
 #include <sys/time.h>
+#include <math.h>
 
 #include "op_pe_utils.h"
 #include "ocount_counter.h"
@@ -42,6 +43,12 @@ bool use_cpu_minus_one = false;
 std::vector<operf_event_t> events;
 op_cpu cpu_type;
 
+#define OCOUNT_MSECS_PER_SEC 1000
+// Current implementation supports a display interval of 100 ms
+#define OCOUNT_DSP_INTVL_MSECS 100
+#define OCOUNT_DSP_INTVLS_PER_SEC (OCOUNT_MSECS_PER_SEC/OCOUNT_DSP_INTVL_MSECS)
+#define OCOUNT_NSECS_PER_MSEC 1000000
+#define OCOUNT_NSECS_PER_DSP_INTVL (OCOUNT_NSECS_PER_MSEC * OCOUNT_DSP_INTVL_MSECS)
 
 static char * app_name_SAVE = NULL;
 static char ** app_args = NULL;
@@ -75,7 +82,7 @@ vector<int> cpus;
 string outfile;
 bool separate_cpu;
 bool separate_thread;
-vector<string> evts;
+set<string> evts;
 bool csv_output;
 long display_interval;
 long num_intervals;
@@ -156,7 +163,7 @@ static void __print_usage_and_exit(const char * extra_msg)
 {
 	if (extra_msg)
 		cerr << extra_msg << endl;
-	cerr << "usage: ocount [ options ] [ --system-wide | -p <pids> | -r <tids> | -C <cpus> [ command [ args ] ] ]" << endl;
+	cerr << "usage: ocount [ options ] [ --system-wide | -p <pids> | -r <tids> | -C <cpus> | [ command [ args ] ] ]" << endl;
 	cerr << "See ocount man page for details." << endl;
 	exit(EXIT_FAILURE);
 }
@@ -221,12 +228,12 @@ bool start_counting(void)
 	if (startApp) {
 		if (pipe(app_ready_pipe) < 0 || pipe(start_app_pipe) < 0) {
 			perror("Internal error: ocount-record could not create pipe");
-			_exit(EXIT_FAILURE);
+			return false;
 		}
 		app_PID = fork();
 		if (app_PID < 0) {
 			perror("Internal error: fork failed");
-			_exit(EXIT_FAILURE);
+			return false;
 		} else if (app_PID == 0) { // child process for exec'ing app
 			run_app();
 		}
@@ -254,7 +261,7 @@ bool start_counting(void)
 		cverb << vdebug << "telling child to start app" << endl;
 		if (write(start_app_pipe[1], &startup, sizeof(startup)) < 0) {
 			perror("Internal error on start_app_pipe");
-			return -1;
+			return false;
 		}
 		app_started = true;
 	}
@@ -300,7 +307,7 @@ static void do_results(ostream & out)
 	try {
 		orecord->output_results(out, ocount_options::separate_cpu | ocount_options::separate_thread,
 		                        ocount_options::csv_output);
-	} catch (runtime_error e) {
+	} catch (const runtime_error & e) {
 		cerr << "Caught runtime error from ocount_record::output_results" << endl;
 		cerr << e.what() << endl;
 		cleanup();
@@ -349,10 +356,55 @@ end_code_t _wait_for_app(ostream & out)
 		long number_intervals = ocount_options::num_intervals;
 		do {
 			struct timeval mytime;
-			int countdown = ocount_options::display_interval;
+			unsigned int countdown, dsp_intvl_in_100ms_units = 0;
+			struct timespec ts_req;
+
+			/* The display_interval is in milliseconds; but at this time, we only allow
+			 * 100ms granularity. */
+			dsp_intvl_in_100ms_units = (int)round(((double)ocount_options::display_interval/
+					OCOUNT_DSP_INTVL_MSECS));
+			if (dsp_intvl_in_100ms_units == 0)
+				dsp_intvl_in_100ms_units = 1; // special case of rounding up; prevent 0 time interval
+			countdown = dsp_intvl_in_100ms_units;
+			cverb << vdebug << "Actual display interval used: " << dsp_intvl_in_100ms_units
+					<< "x100ms" << endl;
+			ts_req.tv_nsec = countdown * OCOUNT_NSECS_PER_DSP_INTVL;
+			ts_req.tv_sec = 0;
 			while (countdown) {
-				sleep(1);
-				if (--countdown == 0) {
+				/* We want to avoid the scenario where, say, the user requests a
+				 * 10 second time interval and the app being counted ends
+				 * immediately after we call sleep().  If we called sleep() with
+				 * the full time interval, we'd be sleeping unnecessarily for
+				 * 10 seconds.  So, for any time interval of one second or longer,
+				 * we do a one second sleep(), wake up and check if the app is
+				 * still alive.
+				 */
+				if (countdown >= OCOUNT_DSP_INTVLS_PER_SEC) {
+					countdown -= OCOUNT_DSP_INTVLS_PER_SEC;
+					sleep(1);
+					if (countdown && (countdown < OCOUNT_DSP_INTVLS_PER_SEC)) {
+						/* The next time through the loop, we'll be taking the
+						 * 'else' leg of this if-statement -- i.e., we will have
+						 * finished the specified time interval, modulo
+						 * OCOUNT_DSP_INTVLS_PER_SEC. So we set sleep time
+						 * accordingly and force dsp_intvl_in_100ms_units to be
+						 * equal to the current countdown value so the "-=" op
+						 * below makes the countdown finally zero.
+						 */
+						ts_req.tv_nsec = countdown * OCOUNT_NSECS_PER_DSP_INTVL;
+						dsp_intvl_in_100ms_units  = countdown;
+					}
+
+				} else {
+					/* We don't bother keeping track of remaining time, since signals delivered
+					 * to this process will be rare (see nanosleep man page). The real time (in
+					 * the supported granularity of 100 ms) is printed out for each time interval,
+					 * so we leave it to the user to note any time gaps.
+					 */
+					(void)nanosleep(&ts_req, NULL);
+					countdown -= dsp_intvl_in_100ms_units;
+				}
+				if (countdown == 0) {
 					if (gettimeofday(&mytime, NULL) < 0) {
 						cleanup();
 						perror("gettimeofday");
@@ -362,7 +414,15 @@ end_code_t _wait_for_app(ostream & out)
 						out << endl << "Current time (seconds since epoch): ";
 					else
 						out << endl << "timestamp,";
-					out << dec << mytime.tv_sec;
+					if (dsp_intvl_in_100ms_units % OCOUNT_DSP_INTVLS_PER_SEC) {
+						int tenths_secs = (int)round(((double)mytime.tv_usec/100000));
+						if (tenths_secs == 10)
+							out << dec << mytime.tv_sec + 1 << "." << "0";
+						else
+							out << dec << mytime.tv_sec << "." << tenths_secs;
+					} else {
+						out << dec << mytime.tv_sec;
+					}
 					do_results(out);
 				}
 				wait_rc = waitpid(app_PID, &waitpid_status, WNOHANG);
@@ -397,11 +457,10 @@ static end_code_t _run(ostream & out)
 		if (!start_counting()) {
 			return PERF_RECORD_ERROR;
 		}
-	} catch (runtime_error e) {
+	} catch (const runtime_error & e) {
 		cerr << "Caught runtime error while setting up counters" << endl;
 		cerr << e.what() << endl;
-		cleanup();
-		exit(EXIT_FAILURE);
+		return PERF_RECORD_ERROR;
 	}
 	// parent continues here
 	if (startApp)
@@ -415,8 +474,20 @@ static end_code_t _run(ostream & out)
 		if (ocount_options::display_interval) {
 			long number_intervals = ocount_options::num_intervals;
 			struct timeval mytime;
+			/* The display_interval is in milliseconds; but at this time, we only allow
+			 * 100ms granularity. */
+			struct timespec ts_req;
+			unsigned int dsp_intvl_in_100ms_units = (int)round(((double)ocount_options::display_interval/
+					OCOUNT_DSP_INTVL_MSECS));
+			if (dsp_intvl_in_100ms_units == 0)
+				dsp_intvl_in_100ms_units = 1; // special case of rounding up; prevent 0 time interval
+			cverb << vdebug << "Actual display interval used: " << dsp_intvl_in_100ms_units
+					<< "x100ms" << endl;
+			// 10 dsp_intvl_in_100ms_units is one second, so we set ts_req accordingly
+			ts_req.tv_sec = dsp_intvl_in_100ms_units/10;
+			ts_req.tv_nsec = (dsp_intvl_in_100ms_units % 10) * OCOUNT_NSECS_PER_DSP_INTVL;
 			while (!stop) {
-				sleep(ocount_options::display_interval);
+				(void)nanosleep(&ts_req, NULL);
 				if (gettimeofday(&mytime, NULL) < 0) {
 					cleanup();
 					perror("gettimeofday");
@@ -426,7 +497,15 @@ static end_code_t _run(ostream & out)
 					out << endl << "Current time (seconds since epoch): ";
 				else
 					out << endl << "t:";
-				out << dec << mytime.tv_sec;
+				if (dsp_intvl_in_100ms_units % OCOUNT_DSP_INTVLS_PER_SEC) {
+					int tenths_secs = (int)round(((double)mytime.tv_usec/100000));
+					if (tenths_secs == 10)
+						out << dec << mytime.tv_sec + 1 << "." << "0";
+					else
+						out << dec << mytime.tv_sec << "." << tenths_secs;
+				} else {
+					out << dec << mytime.tv_sec;
+				}
 				do_results(out);
 				if (--number_intervals == 0)
 					stop = true;
@@ -482,15 +561,17 @@ static void _parse_time_interval(void)
 	char * endptr;
 	char * num_intervals, * interval = strtok(optarg, ":");
 	ocount_options::display_interval = strtol(interval, &endptr, 10);
-	if ((endptr >= interval) && (endptr <= (interval + strlen(interval) - 1))) {
-			__print_usage_and_exit("ocount: Invalid numeric value for num_seconds.");
-	}
+	if (((endptr >= interval) && (endptr <= (interval + strlen(interval) - 1))) ||
+			(ocount_options::display_interval < 0))
+		__print_usage_and_exit("ocount: Invalid numeric value for interval_length.");
+
 	// User has specified num_intervals: e.g., '-i 5:10'
 	num_intervals = strtok(NULL, ":");
 	if (num_intervals) {
 		ocount_options::num_intervals = strtol(num_intervals, &endptr, 10);
-		if ((endptr >= num_intervals) && (endptr <= (num_intervals + strlen(num_intervals) - 1)))
-				__print_usage_and_exit("ocount: Invalid numeric value for num_intervals.");
+		if (((endptr >= num_intervals) && (endptr <= (num_intervals + strlen(num_intervals) - 1))) ||
+				(ocount_options::num_intervals < 0))
+			__print_usage_and_exit("ocount: Invalid numeric value for num_intervals.");
 	}
 }
 
@@ -549,7 +630,7 @@ static int _process_ocount_and_app_args(int argc, char * const argv[])
 		case 'e':
 			event = strtok(optarg, ",");
 			do {
-				ocount_options::evts.push_back(event);
+				ocount_options::evts.insert(event);
 			} while ((event = strtok(NULL, ",")));
 			break;
 		case 'f':
@@ -720,9 +801,9 @@ static void process_args(int argc, char * const argv[])
 
 	if (ocount_options::evts.empty()) {
 		// Use default event
-		op_pe_utils::op_get_default_event();
+		op_pe_utils::op_get_default_event(false);
 	} else  {
-		op_pe_utils::op_process_events_list(ocount_options::evts);
+		op_pe_utils::op_process_events_list(ocount_options::evts, false, false);
 	}
 	cverb << vdebug << "Number of events passed is " << events.size() << endl;
 	return;
@@ -755,7 +836,10 @@ int main(int argc, char * const argv[])
 		}
 	}
 	if (rc == EBUSY)
-		cerr << "Performance monitor unit is busy.  Do 'opcontrol --deinit' and try again." << endl;
+		cerr << "Performance monitor unit is busy.  Ensure that no other profilers are running on the system." << endl
+		     << "Note: For example, the obsolete opcontrol profiler (available in earlier oprofile releases)" << endl
+		     << "does not allow other performance tools to run simultaneously. To check for this, look for the" << endl
+		     << "'oprofiled' process using the 'ps' command." << endl;
 	else if (rc == ENOSYS)
 		cerr << "Your kernel does not implement a required syscall"
 		     << " for the ocount program." << endl;
@@ -769,10 +853,26 @@ int main(int argc, char * const argv[])
 		exit(1);
 
 	cpu_type = op_get_cpu_type();
+
+	if (cpu_type == CPU_NO_GOOD) {
+		cerr << "Unable to ascertain cpu type.  Exiting." << endl;
+		cleanup();
+		exit(1);
+	}
+
+	if (cpu_type == CPU_TIMER_INT) {
+		cerr << "CPU type 'timer' was detected, but ocount does not support 'timer' as a cpu type." << endl
+		     << "Ensure the obsolete opcontrol profiler (available in pre-1.0 oprofile releases)" << endl
+		     << "is not running on the system.  To check for this, look for the file" << endl
+		     << "/dev/oprofile/cpu_type; if this file exists, locate the pre-1.0 oprofile" << endl
+		     << "installation, and use its 'opcontrol' command with the --deinit option." << endl;
+		cleanup();
+		exit(1);
+	}
 	cpu_speed = op_cpu_frequency();
 	try {
 		process_args(argc, argv);
-	} catch (runtime_error e) {
+	} catch (const runtime_error & e) {
 		cerr << "Caught runtime error while processing args" << endl;
 		cerr << e.what() << endl;
 		cleanup();
@@ -791,12 +891,6 @@ int main(int argc, char * const argv[])
 		exit(1);
 	}
 
-	if (cpu_type == CPU_NO_GOOD) {
-		cerr << "Unable to ascertain cpu type.  Exiting." << endl;
-		cleanup();
-		exit(1);
-	}
-
 	if (!ocount_options::outfile.empty()) {
 		outfile.open(ocount_options::outfile.c_str());
 	}
